@@ -9,7 +9,8 @@ from .common import (
     get_pyodide_root,
     get_unisolated_packages,
 )
-from .io import MetaConfig
+from .logger import logger
+from .recipe import load_all_recipes
 
 
 def make_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
@@ -30,9 +31,9 @@ def copy_xbuild_files(xbuildenv_path: Path) -> None:
     # pip install -t $HOSTSITEPACKAGES -r requirements.txt
     # cp site-packages-extras $HOSTSITEPACKAGES
     site_packages_extras = xbuildenv_path / "site-packages-extras"
-    for pkg in (PYODIDE_ROOT / "packages").glob("**/meta.yaml"):
-        config = MetaConfig.from_yaml(pkg)
-        xbuild_files = config.build.cross_build_files
+    recipes = load_all_recipes(PYODIDE_ROOT / "packages")
+    for recipe in recipes.values():
+        xbuild_files = recipe.build.cross_build_files
         for path in xbuild_files:
             target = site_packages_extras / path
             target.parent.mkdir(parents=True, exist_ok=True)
@@ -56,7 +57,6 @@ def copy_wasm_libs(xbuildenv_path: Path) -> None:
         Path("Makefile.envs"),
         wasm_lib_dir / "cmake",
         Path("dist/repodata.json"),
-        Path("dist/pyodide_py.tar"),
         Path("dist/python"),
     ]
     to_copy.extend(
@@ -103,7 +103,7 @@ def main(args: argparse.Namespace) -> None:
         encoding="utf8",
     )
     if res.returncode != 0:
-        print("Failed to install node-fetch:")
+        logger.error("Failed to install node-fetch:")
         exit_with_stdio(res)
 
     res = subprocess.run(
@@ -112,7 +112,7 @@ def main(args: argparse.Namespace) -> None:
         encoding="utf8",
     )
     if res.returncode != 0:
-        print("Failed to run pip freeze:")
+        logger.error("Failed to run pip freeze:")
         exit_with_stdio(res)
 
     (xbuildenv_path / "requirements.txt").write_text(res.stdout)
