@@ -1,12 +1,14 @@
 import argparse
 import json
+import os
 import shutil
 import subprocess
 from pathlib import Path
 from urllib.request import urlopen, urlretrieve
 
-from .common import exit_with_stdio, get_make_flag, get_pyodide_root
+from .common import exit_with_stdio, get_make_flag
 from .create_pypa_index import create_pypa_index
+from .logger import logger
 
 
 def make_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
@@ -19,37 +21,47 @@ def make_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
         "Note: this is a private endpoint that should not be used outside of the Pyodide Makefile."
     )
     parser.add_argument("--download", action="store_true", help="Download xbuild env")
+    parser.add_argument("--url", help="URL to download xbuild env from", default=None)
     parser.add_argument("xbuildenv", type=str, nargs=1)
     return parser
 
 
-def download_xbuildenv(version: str, xbuildenv_path: Path) -> None:
+def download_xbuildenv(
+    version: str, xbuildenv_path: Path, *, url: str | None = None
+) -> None:
     from shutil import rmtree, unpack_archive
     from tempfile import NamedTemporaryFile
 
-    print("Downloading xbuild environment")
+    logger.info("Downloading xbuild environment")
     rmtree(xbuildenv_path, ignore_errors=True)
+
+    xbuildenv_url = (
+        url
+        or f"https://github.com/pyodide/pyodide/releases/download/{version}/xbuildenv-{version}.tar.bz2"
+    )
     with NamedTemporaryFile(suffix=".tar") as f:
         urlretrieve(
-            f"https://github.com/pyodide/pyodide/releases/download/{version}/xbuildenv-{version}.tar.bz2",
+            xbuildenv_url,
             f.name,
         )
         unpack_archive(f.name, xbuildenv_path)
 
 
 def install_xbuildenv(version: str, xbuildenv_path: Path) -> None:
-    print("Installing xbuild environment")
+    logger.info("Installing xbuild environment")
+
     xbuildenv_path = xbuildenv_path / "xbuildenv"
-    pyodide_root = get_pyodide_root()
     xbuildenv_root = xbuildenv_path / "pyodide-root"
-    host_site_packages = xbuildenv_root / Path(
-        get_make_flag("HOSTSITEPACKAGES")
-    ).relative_to(pyodide_root)
+
+    os.environ["PYODIDE_ROOT"] = str(xbuildenv_root)
+
+    host_site_packages = Path(get_make_flag("HOSTSITEPACKAGES"))
     host_site_packages.mkdir(exist_ok=True, parents=True)
     result = subprocess.run(
         [
             "pip",
             "install",
+            "--no-user",
             "-t",
             host_site_packages,
             "-r",
@@ -66,8 +78,8 @@ def install_xbuildenv(version: str, xbuildenv_path: Path) -> None:
         xbuildenv_path / "site-packages-extras", host_site_packages, dirs_exist_ok=True
     )
     cdn_base = f"https://cdn.jsdelivr.net/pyodide/v{version}/full/"
-    if (xbuildenv_root / "repodata.json").exists():
-        repodata_bytes = (xbuildenv_root / "repodata.json").read_bytes()
+    if (repodata_json := xbuildenv_root / "dist" / "repodata.json").exists():
+        repodata_bytes = repodata_json.read_bytes()
     else:
         repodata_url = cdn_base + "repodata.json"
         with urlopen(repodata_url) as response:
@@ -77,11 +89,33 @@ def install_xbuildenv(version: str, xbuildenv_path: Path) -> None:
     create_pypa_index(repodata["packages"], xbuildenv_root, cdn_base)
 
 
-def main(args: argparse.Namespace) -> None:
+def install(path: Path, *, download: bool = False, url: str | None = None) -> None:
+    """
+    Install cross-build environment.
+
+    Parameters
+    ----------
+    path
+        A path to the cross-build environment.
+    download
+        Whether to download the cross-build environment before installing it.
+    url
+        URL to download the cross-build environment from. This is only used
+        if `download` is True. The URL should point to a tarball containing
+        the cross-build environment. If not specified, the corresponding
+        release on GitHub is used.
+
+        Warning: if you are downloading from a version that is not the same
+        as the current version of pyodide-build, make sure that the cross-build
+        environment is compatible with the current version of Pyodide.
+    """
     from . import __version__
 
-    xbuildenv_path = Path(args.xbuildenv[0])
     version = __version__
-    if args.download:
-        download_xbuildenv(version, xbuildenv_path)
-    install_xbuildenv(version, xbuildenv_path)
+    if download:
+        download_xbuildenv(version, path, url=url)
+    install_xbuildenv(version, path)
+
+
+def main(args: argparse.Namespace) -> None:
+    install(Path(args.xbuildenv[0]), download=args.download, url=args.url)
