@@ -30,7 +30,7 @@ def initialize_pyodide_root(*, quiet: bool = False) -> None:
     if "PYODIDE_ROOT" in os.environ:
         return
     try:
-        os.environ["PYODIDE_ROOT"] = str(search_pyodide_root(__file__))
+        os.environ["PYODIDE_ROOT"] = str(search_pyodide_root(Path.cwd()))
         return
     except FileNotFoundError:
         pass
